/*
Copyright 2023 by Valentina Gliozzi, Gian Luca Pozzato, Gabriele Tessore, Alberto Valese

    This file is part of CLean.

    CLean is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    CLean is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CLean.  If not, see <http://www.gnu.org/licenses/>.
*/
:- use_module(library(dialect/sicstus/timeout), [time_out/3]).
:- dynamic catch/1.

/* Module for statistics */

/* Consult the file of the theorem prover (e.g. ['ckcs.pl'])
   and then run:
   - stats_on_valid(Ms,Perc)   for testing CLean on
                                 valid formulas 
                                 Ms: time limit in ms
                                 Perc: percentage of successes
   - stats_on_random(Ms,[PS,PF,PTO])   
                                for testing CLean on
                                 ranomly generated formulas 
                                 Ms: time limit in ms
                                 PS: percentage of successes
                                 PF: percentage of failures
                                 PTO: percentage of time outs
*/

stats_on_random(N,Ms,[SuccP,FailP,TOP]):-
    generateFormulas(N,List),
    length(List,N),
    loop_on_fmls(Ms,List,[Succ,Fail,TO]),
    SuccP is 100*Succ/N,
    FailP is 100*Fail/N,
    TOP is 100*TO/N.

stats_on_valid(Ms,Perc):-
    validFormulas(List),
    length(List,N),
    easy_loop_on_fmls(Ms,List,Result),
    Perc is 100*Result/N.

loop_on_fmls(_,[],[0,0,0]):-!.
loop_on_fmls(Ms,[Fml|Tail],[NewSucc,NewFail,NewTO]):-
    (catch(call_with_time_limit(Ms,pr(Fml)),Exception,true) ->
        (var(Exception) -> 
            (Time_Out = 1,
            Fail = 0,
            Succ = 0)
            ;
            (Time_Out = 0,
            Fail = 0,
            Succ = 1)
        )
        ;
        (Time_Out = 0,
            Fail = 1,
            Succ = 0)
    ),
    loop_on_fmls(Ms,Tail,[TailSucc,TailFail,TailTO]),
    NewSucc is Succ + TailSucc,
    NewFail is Fail + TailFail,
    NewTO is Time_Out + TailTO.


easy_loop_on_fmls(_,[],0):-!.
easy_loop_on_fmls(Ms,[Fml|Tail],Succ):-
    call_with_time_limit(Ms,pr(Fml)),!,
    easy_loop_on_fmls(Ms,Tail,SuccTail),
    Succ is 1 + SuccTail.
easy_loop_on_fmls(Ms,[_|Tail],Succ):-
    easy_loop_on_fmls(Ms,Tail,Succ).



validFormulas([
(a and b) -> (a => b),
(a => (b and c))->(a => b),
(a and b) -> (a => b),
(a and b1 and b2) -> ((a => b1) and (a => b2)) ,
(a and b1 and b2 and b3) -> ((a => b1) and (a => b2) and (a => b3)) ,
(a and b1 and b2 and b3 and b4) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4)) ,
(a and b1 and b2 and b3 and b4 and b5) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42 and b43) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42) and (a => b43)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42 and b43 and b44) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42) and (a => b43) and (a => b44)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42 and b43 and b44 and b45) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42) and (a => b43) and (a => b44) and (a => b45)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42 and b43 and b44 and b45 and b46) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42) and (a => b43) and (a => b44) and (a => b45) and (a => b46)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42 and b43 and b44 and b45 and b46 and b47) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42) and (a => b43) and (a => b44) and (a => b45) and (a => b46) and (a => b47)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42 and b43 and b44 and b45 and b46 and b47 and b48) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42) and (a => b43) and (a => b44) and (a => b45) and (a => b46) and (a => b47) and (a => b48)) ,
(a and b1 and b2 and b3 and b4 and b5 and b6 and b7 and b8 and b9 and b10 and b11 and b12 and b13 and b14 and b15 and b16 and b17 and b18 and b19 and b20 and b21 and b22 and b23 and b24 and b25 and b26 and b27 and b28 and b29 and b30 and b31 and b32 and b33 and b34 and b35 and b36 and b37 and b38 and b39 and b40 and b41 and b42 and b43 and b44 and b45 and b46 and b47 and b48 and b49) -> ((a => b1) and (a => b2) and (a => b3) and (a => b4) and (a => b5) and (a => b6) and (a => b7) and (a => b8) and (a => b9) and (a => b10) and (a => b11) and (a => b12) and (a => b13) and (a => b14) and (a => b15) and (a => b16) and (a => b17) and (a => b18) and (a => b19) and (a => b20) and (a => b21) and (a => b22) and (a => b23) and (a => b24) and (a => b25) and (a => b26) and (a => b27) and (a => b28) and (a => b29) and (a => b30) and (a => b31) and (a => b32) and (a => b33) and (a => b34) and (a => b35) and (a => b36) and (a => b37) and (a => b38) and (a => b39) and (a => b40) and (a => b41) and (a => b42) and (a => b43) and (a => b44) and (a => b45) and (a => b46) and (a => b47) and (a => b48) and (a => b49)) 
]).



generateFormulas(0,[]):-!.
generateFormulas(N,[Fml|Tail]):-
    random(0,7,Degree),
    generateSingleFormula(Degree,Fml),
    N1 is N-1,
    generateFormulas(N1,Tail).

generateSingleFormula(0,A):-!,randomletter(A).
generateSingleFormula(Degree,F):-
    random(0,5,Connective),
    Connective>0,!,
    Deg is Degree-1,
    generateSingleFormula(Deg,A),
    generateSingleFormula(Deg,B),
    buildFormula(Connective,A,B,F).
generateSingleFormula(Degree,F):-
    Deg is Degree-1,
    generateSingleFormula(Deg,SubF),
    buildNegation(SubF,F).

randomletter(A):-random(0,3,N),pickLetter(N,A).
pickLetter(0,a):-!.
pickLetter(1,b):-!.
pickLetter(2,c):-!.
buildNegation(F,neg F):-!.
buildFormula(1,A,B,A or B):-!.
buildFormula(2,A,B,A and B):-!.
buildFormula(3,A,B,A -> B):-!.
buildFormula(4,A,B,A => B):-!.
